# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.2.0

import enum
import typing
import uuid as _uuid

import winrt.system
from winrt import _winrt_windows_ui_popups

__all__ = [
    "MessageDialogOptions",
    "Placement",
    "MessageDialog",
    "PopupMenu",
    "UICommand",
    "UICommandSeparator",
    "IUICommand",
    "UICommandInvokedHandler",
]

class MessageDialogOptions(enum.IntFlag):
    NONE = 0x0
    ACCEPT_USER_INPUT_AFTER_DELAY = 0x1

class Placement(enum.IntEnum):
    DEFAULT = 0
    ABOVE = 1
    BELOW = 2
    LEFT = 3
    RIGHT = 4

MessageDialog = _winrt_windows_ui_popups.MessageDialog
PopupMenu = _winrt_windows_ui_popups.PopupMenu
UICommand = _winrt_windows_ui_popups.UICommand
UICommandSeparator = _winrt_windows_ui_popups.UICommandSeparator
IUICommand = _winrt_windows_ui_popups.IUICommand
UICommandInvokedHandler = typing.Callable[[typing.Optional[IUICommand]], None]
